<?php

namespace App\Services;

use App\Models\Log as ModelsLog;
use App\Services\Error\SetLogs;
use Illuminate\Http\Client\RequestException;
use Illuminate\Http\Client\Response;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;

class CurlToLaravel
{
    protected $transaction_id, $logs;

    public function __construct()
    {
        $this->logs = new SetLogs();
    }

    public function sendSuccess($transaction, $gateway, $fee)
    {
        //curl to laravel when verify is success and return laravel response.
        $curlSuccess = Http::withHeaders([
            'Authorization' => $transaction->authorization,
            'PROXY-TOKEN' => config('proxy.reseller_token'),
            "Accept" => "application/json"
        ])->post("https://" . config('proxy.target_url') . '/api/v3/pay/success', [
            'token' => $transaction->token,
            'tracking_code' => $transaction->tracking_code,
            'gateway' => $gateway,
            'fee' => $fee,
        ])->throw(function (Response $response, RequestException $exception) {
            Log::error('send failed curl was failed because : ' . $exception->getMessage());
            $this->logs->setLogs(ModelsLog::LEVEL['prepare_payment'], $exception->getMessage(), $exception->getTraceAsString());
        })->collect();

        $this->setLogs($curlSuccess, $transaction->id);
    }

    public function sendFailed($transaction)
    {
        //curl to laravel when payment or verify is failed and return laravel response.
        $curlFail = Http::withHeaders([
            'PROXY-TOKEN' => config('proxy.reseller_token'),
            'Authorization' => $transaction->authorization,
            "Accept" => "application/json"
        ])->post("https://" . config('proxy.target_url') . '/api/v3/pay/failed', [
            'token' => $transaction->token,
            'note' => $transaction->note,
        ])->throw(function (Response $response, RequestException $exception) {

            Log::error('send failed curl was failed because : ' . $exception->getMessage());
            $this->logs->setLogs(ModelsLog::LEVEL['prepare_payment'], $exception->getMessage(), $exception->getTraceAsString());
        })->collect();

        $this->setLogs($curlFail, $transaction->id, 'failed');
    }

    private function setLogs($curl, $transaction_id, $success = 'success')
    {
        if ($curl['status'] !== 'success') {
            $this->logs->setLogs(ModelsLog::LEVEL[$success . '_curl'], $curl['message'], null, null, $transaction_id); //ask about 4th parameter 
            Log::error('send ' . $success . ' curl was failed because : ' . $curl['message']);
        }
    }
}
