<?php

namespace App\Services;

use App\Models\Log as ModelsLog;
use App\Models\Transaction;
use App\Services\Error\SetLogs;
use Exception;
use Illuminate\Support\Facades\Log;
use Shetabit\Multipay\Payment;
use Shetabit\Multipay\Exceptions\InvalidPaymentException;
use Shetabit\Multipay\Invoice;

class Gateway
{
    protected $curl, $logs;

    public function __construct()
    {
        $this->curl = new CurlToLaravel();

        $this->logs = new SetLogs();
    }
    public function prepare($data)
    {
        $transaction = Transaction::create([
            'authorization' => request()->header('Authorization'), //"210|1MeWhBBdfrwsp0nJ1ptnbPoQjvM3kD9BTlna1fOT2d62334f",
            'amount' => $data['amount'],
            'user_id' => $data['user']['id'],
            'user_name' => $data['user']['name'],
            'user_family' => $data['user']['family'],
            'user_email' => $data['user']['email'],
            'user_mobile' => $data['user']['phone'],
            'status' => Transaction::STATUS['pending'],
            'token' => $data['token'],
            'uuid' => uniqid(),
            'gateway' => json_encode(config('payment.default'))
        ]);

        return ['url' => route('pay', ['uuid' => $transaction->uuid])];
    }

    public function pay()
    {
        try {
            $transaction = Transaction::where('uuid', request('uuid'))->where('status', Transaction::STATUS['pending'])->first();
            throw_if(empty($transaction),  Exception::class, 'transaction with uuid : ' . request('uuid') . ' not found!');
            $emptyKeys = array_keys(config('payment.drivers.' . config('payment.default')), '', true);

            throw_if(!empty($emptyKeys),  Exception::class, "expected send " . json_encode($emptyKeys) . "key(s), but not send.");

            $invoice = (new Invoice);
            $invoice->amount($transaction->amount);
            return (new Payment(config('payment')))->callbackUrl(route('verify', ['uuid' => $transaction->uuid]))->purchase(
                $invoice,
                function ($driver, $transactionId) use ($transaction) {
                    $transaction->terminal_id = $transactionId;
                    $transaction->status = Transaction::STATUS['pending_pay'];
                    $transaction->save();
                }
            )->pay()->render();
        } catch (\Throwable $exception) {

            if (isset($transaction)) {

                $transaction->update([
                    'status_before_failing' => $transaction->status,
                    'status' => Transaction::STATUS['failed'],
                    'note' => $exception->getMessage(),
                    'gateway' => '{}',
                ]);
                //* curl to laravel payment is failed.
                $this->curl->sendFailed($transaction);

                return redirect(config('setting.spa_callback_url') . '?token=' . $transaction->token);
            }

            Log::error('payment failed : ' . $exception->getMessage());

            $this->logs->setLogs(ModelsLog::LEVEL['pending_pay'], $exception->getMessage(), $exception->getTraceAsString(), null, isset($transaction) ? $transaction->id : null); //ask about 4th parameter 

            return redirect(config('setting.spa_callback_url') . '?token=');
        }
    }

    public function verify()
    {
        try {
            $payment = Transaction::where('uuid', request('uuid'))->where('status', Transaction::STATUS['pending_pay'])->first();

            throw_if(empty($payment),  Exception::class, 'transaction with uuid : ' . request('uuid') . ' not found!');

            $payment->status = Transaction::STATUS['pending_verify'];
            $payment->save();

            $receipt = (new Payment(config('payment')))->amount($payment->amount)->transactionId($payment->terminal_id)->verify();

            $payment->update([
                'status' => Transaction::STATUS['paid'],
                'tracking_code' => $receipt->getReferenceId()  ,
            ]);

            //* curl to laravel payment is success and user must be increase charge balance
            $this->curl->sendSuccess($payment, $payment->gateway, 0);
            $url = is_null($payment->callback) ? config('setting.spa_callback_url') : 'http://' . $payment->callback;
            return redirect($url . '?token=' . $payment->token);
            // return redirect(config('setting.spa_callback_url') . '?token=' . $payment->token );
        } catch (InvalidPaymentException | \Throwable $exception) {

            if (isset($payment)) {

                $payment->update([
                    'status_before_failing' => $payment->status,
                    'status' => Transaction::STATUS['failed'],
                    'note' => $exception->getMessage(),
                ]);

                //* curl to laravel payment is failed.
                $this->curl->sendFailed($payment);

                $url = is_null($payment->callback) ? config('setting.spa_callback_url') : 'http://' . $payment->callback;
                return redirect($url . '?token=' . $payment->token);
                return redirect(config('setting.spa_callback_url') . '?token=' . $payment->token);
            }

            Log::error('verify failed : ' . $exception->getMessage());

            $this->logs->setLogs(ModelsLog::LEVEL['pending_verify'], $exception->getMessage(), $exception->getTraceAsString(),  null, isset($payment) ? $payment->id : null); //ask about 4th parameter 

            return redirect(config('setting.spa_callback_url'));
        }
    }
}
