<?php

namespace App\Services\Payment;

use Illuminate\Support\Facades\Log;

class Gateway
{
    protected $true_response_keys;

    public function __construct()
    {
        $this->true_response_keys = ['token', 'gateway_config', 'gateway_name'];
    }

    public function neededConfigs($gateway_name)
    {
        //return gateway configs from config file that fields values are empty
        return $this->getEmptyKeys($this->getConfigs($gateway_name));
    }

    public function getConfigs($gateway_name = null)
    {
        if (!is_null($gateway_name)) {
            return config('payment.drivers.' . $gateway_name);
        } else {
            return config('payment');
        }
    }

    public function getDriversName()
    {
        return array_keys($this->getConfigs()['drivers']);
    }

    public function getEmptyKeys($input)
    {
        return array_keys($input, '', true);
    }

    public function responseKeysChecker($input)
    {
        $different_keys = array_diff($this->true_response_keys, array_keys($input));

        if (!empty($different_keys)) {
            Log::error('wrong response. missing : ' . json_encode(array_values($different_keys)));
            return false;
        }

        return true;
    }

    public function setGatewayConfig($gateway_name, $gateway_config)
    {
        return [
            'default' => $gateway_name,
            'drivers' => [$gateway_name => array_merge($this->getConfigs($gateway_name), $gateway_config)],
            'map' => [$gateway_name => config('payment.map.' . $gateway_name)]
        ];
    }

    public function setErrorForEmptyKeys($input)
    {
        if (!is_null($this->getEmptyKeys($input))) {
            Log::error('missing param(s) : ' . json_encode($this->getEmptyKeys($input)));
            return false;
        }
        return true;
    }

    public function isExistGateway($gateway)
    {
        //* if there input gateway in drivers list return true
        return in_array($gateway, $this->getDriversName());
    }
}
